'From Squeak3.2gamma of 15 January 2002 [latest update: #4743] on 20 February 2002 at 12:35:52 pm'!
"Change Set:		VMMaker-Ross
Date:			20 February 2002
Author:			Ross Boylan

This change set is a modification to Tim Rowledge's VMMaker-3-2-version5, and should be filed in after it.  This targets MSVC rather than GCC, and in fact will break the latter.  It should be used with the updated .dsp and other files I provide elsewhere.

Instead of copying the basic files, this installs references to them in the .dsp file, which is rewritten.  This is still designed only for internal plugins, and does not support MPEG3 plugin.

The change in Interpreter was done when I was considering tracking all file writes, which I backed off of.  It adds a little encapsulation, so I left it.

I anticipate this will change massively before it is final.  Partly for that reason, I'm holding off making gcc work.  There seems no point in getting it to work and then immediately breaking things by going to separate directories for plugins."!

VMMakerWithFileCopying subclass: #Win32VMMaker
	instanceVariableNames: 'cFiles hFiles projectFileSrc hDirs projectFileDst '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VMConstruction-Building'!

!Win32VMMaker commentStamp: 'RB 2/19/2002 23:08' prior: 0!
Make Win32 source files.

This has two tweaks.

We put all the files in one directory.

We build up the MSVC project file by hand.

Instance Variables:
projectFileSrc <String>	Name of the template .dsp file
projectFileDst <String>	Name of the .dsp file we will build
cFiles		<OrderedCollection> filenames to link into project
hFiles		<OrderedCollection> filenames to link into project.
hDirs		<Set>		directories of header files.

There are many code paths that generate files, so rather than try to follow them we're going to look at the directory after it's built to make the dsp file.

As this evolves, we will probably stop copying many of the files and link to them directly.
We also will need to develop subprojects or otherwise handle separate directories.


Phase 1: I stop copying the basic c and h files and put links in the .dsp file.  THIS WILL BREAK GCC makefile.
!


!Interpreter class methodsFor: 'translation' stamp: 'RB 2/19/2002 15:53'!
fileOfDirectory: directory
	"Return the name of my output file"
	^directory fullNameFor: 'interp.c'.
! !

!Interpreter class methodsFor: 'translation' stamp: 'RB 2/19/2002 15:54'!
translateInDirectory: directory doInlining: inlineFlag forBrowserPlugin: pluginFlag
	"Translate the Smalltalk description of the virtual machine into C. If inlineFlag is true, small method bodies are inlined to reduce procedure call overhead. On the PPC, this results in a factor of three speedup with only 30% increase in code size. If pluginFlag is true, generate code for an interpreter that runs as a browser plugin (Netscape or IE)."
	"Note: The pluginFlag is meaningless on Windows and Unix. On these platforms Squeak runs as it's own process and doesn't need any special attention from the VMs point of view. Meaning that NONE of the required additional functions will be supported. In other words, the pluginFlag is not needed and not supported."
	"Return the list of exports for this module"

	| doInlining cg exports fileName|
	doInlining _ inlineFlag.
	pluginFlag ifTrue: [doInlining _ true].  "must inline when generating browser plugin"
	Interpreter initialize.
	ObjectMemory initialize.
	GenerateBrowserPlugin _ pluginFlag.
	cg _ CCodeGenerator new initialize.
	cg addClass: Interpreter.
	cg addClass: ObjectMemory.
	Interpreter declareCVarsIn: cg.
	ObjectMemory declareCVarsIn: cg.

	"Get all the named prims from the VM.
	Note: the format of exports is:
		pluginName -> Array of: primitiveName.
	so we can generate a nice table from it."
	exports _ #vm -> cg exportedPrimitiveNames asArray.

	fileName _ self fileOfDirectory: directory.
	cg storeCodeOnFile: fileName doInlining: doInlining.

	^exports! !


!Win32VMMaker methodsFor: 'initialize' stamp: 'RB 2/19/2002 23:14'!
initializeForRun
	"Set up for a generation of files"
	cFiles _ OrderedCollection new.
	hFiles _ OrderedCollection new.
	hDirs _ Set with: (self coreVMDirectory fullName).! !

!Win32VMMaker methodsFor: 'generate sources' stamp: 'RB 2/19/2002 22:52'!
generateEntire
	"This is not the ideal place for the call to makeProjectFile, since other code paths need it too"
	self initializeForRun.
	super generateEntire.
	self makeProjectFile! !

!Win32VMMaker methodsFor: 'private - copying files' stamp: 'RB 2/19/2002 23:12'!
copyFileNamed: srcName to: dstName
	"Catch special handling"
	|ext|
	ext _ FileDirectory extensionFor: srcName.
	ext = 'dsp' ifTrue: [
		projectFileSrc _ srcName.
		projectFileDst _ dstName.
		"We do not copy it at this time"
		^self.
	].
	ext = 'c' ifTrue: [
		cFiles add: srcName.
	] ifFalse: [
	ext = 'h' ifTrue: [
		self noteHeaderFile: srcName.
	] ifFalse: [
		"We do not copy standard h and c files"
		^super copyFileNamed: srcName to: dstName.
	]].! !

!Win32VMMaker methodsFor: 'private - copying files' stamp: 'RB 2/19/2002 21:21'!
makeProjectFile
	"After all other files have been copied, set up the project file"
	|project template line inGroupSection propHeader propEnd ext|
	project _ CrLfFileStream newFileNamed: projectFileDst.
	template _ FileStream readOnlyFileNamed: projectFileSrc.
	line _ template nextLine.
	inGroupSection _ false.
	propHeader _ '# PROP Default_Filter "'.
	propEnd _ '# End Group'.
	[template atEnd] whileFalse: [
		inGroupSection ifFalse: [
			self writeProjectLine: line to: project.
			(line beginsWith: propHeader) ifTrue: [
				inGroupSection _ true.
				ext _ line at: propHeader size + 1.
				ext = $c ifTrue: [
					self writeSourceFilesTo: project ext: 'c'.
				] ifFalse: [
				ext = $h ifTrue: [
					self writeSourceFilesTo: project ext: 'h'.
				] ifFalse: [
					"let unknown sections pass"
					inGroupSection _ false.
				]]].
		] ifTrue: [
			"We are now skipping over a group"
			(line beginsWith: propEnd) ifTrue: [
				project nextPutAll: line; cr.
				inGroupSection _ false.
				].
		].
		template skip: 1.
		line _ template nextLine.
		].
	project close.
	template close.
	! !

!Win32VMMaker methodsFor: 'private - copying files' stamp: 'RB 2/19/2002 23:11'!
noteHeaderFile: name
	"Record presence of header file"
	hFiles add: name.
	hDirs add: (FileDirectory dirPathFor: name).! !

!Win32VMMaker methodsFor: 'private - copying files' stamp: 'RB 2/19/2002 23:16'!
writeProjectLine: line to: project
	"Write out a regular line of the project, substituting include path as necessary"
	|i includeString| 
	includeString _ ' /I "../generated" '.
	i _ line findString: includeString.
	i == 0 ifTrue: [
		project nextPutAll: line; cr.
		^self.
	].

	"Special handling"
	project nextPutAll: (line first: i-1).
	hDirs do: [:aName |
		project
			nextPutAll: ' /I "';
			nextPutAll: aName;
			nextPutAll: '" '.
		].
	project nextPutAll: (line allButFirst: includeString size + i - 1);
		cr.! !

!Win32VMMaker methodsFor: 'private - copying files' stamp: 'RB 2/19/2002 22:57'!
writeSourceFile: name to: project
	"Here we stream out the code to include one files in the .dsp file"
	project nextPutAll: '# Begin Source File'; cr; cr;
		nextPutAll: 'SOURCE='; nextPutAll: name; cr;
		nextPutAll: '# End Source File'; cr.
! !

!Win32VMMaker methodsFor: 'private - copying files' stamp: 'RB 2/19/2002 23:00'!
writeSourceFilesTo: project ext: extension
	"Here we stream out the code to include our files in the .dsp file"
	|dir dirName|
	dir _ self coreVMDirectory.
	dirName _ '.\', self class coreVMDirName, '\'.
	dir fileNames do: [:name |
		|ext|
		ext _ FileDirectory extensionFor: name.
		ext = extension ifTrue: [
			self writeSourceFile: dirName, name to: project.
			].
		].
	extension = 'c' ifTrue: [
		cFiles do: [:name |
			self writeSourceFile: name to: project.
			].
	] ifFalse: [
	extension = 'h' ifTrue: [
		hFiles do: [:name |
			self writeSourceFile: name to: project.
			].
	]].! !

VMMakerWithFileCopying subclass: #Win32VMMaker
	instanceVariableNames: 'projectFileSrc projectFileDst cFiles hFiles hDirs '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VMConstruction-Building'!
